import os
from io import BytesIO

import requests
import torch
from diffusers import EulerAncestralDiscreteScheduler, StableDiffusionXLPipeline
from PIL import Image
from rich import traceback

traceback.install()


scheduler = EulerAncestralDiscreteScheduler()
pipe = StableDiffusionXLPipeline.from_single_file(
    "/workspace/models/marsey-xl.safetensors",
    torch_dtype=torch.bfloat16,
    safety_checker=None,
    scheduler=scheduler,
)
pipe.to("cuda")
pipe.enable_xformers_memory_efficient_attention()


def generate(prompt):
    images = []
    for _ in range(3):
        batch = pipe(prompt=[prompt] * 3, guidance_scale=7.0, num_inference_steps=30)
        images.extend(batch.images)

    return images


def image_grid(images, rows, cols):
    assert len(images) == rows * cols

    width = max(image.width for image in images)
    height = max(image.height for image in images)

    grid = Image.new("RGB", size=(width * cols, height * rows))

    for index, image in enumerate(images):
        row = index // cols
        col = index % cols
        grid.paste(image, (col * width, row * height))

    return grid


def process_queue():
    queue_json = requests.get("https://rdra.ma/marseygen").json()

    for request_id, prompt in queue_json.items():
        images = generate(prompt)
        grid_image = image_grid(images, 3, 3)

        # Save the image to a BytesIO object.
        buffered = BytesIO()
        grid_image.save(buffered, format="WEBP")

        # Reset the buffer position to the beginning.
        buffered.seek(0)

        # Post the image to the server.
        response = requests.post(
            f"https://rdra.ma/marseygen/{request_id}?key={os.getenv('SERVER_KEY')}",
            files={"file": ("image.webp", buffered, "image/webp")},
        )
        print(f"Response for request {request_id}: {response.status_code}")


if __name__ == "__main__":
    process_queue()
